        subroutine rec2ems(iyear,idoy,sec,x_sta,y_sta,z_sta,a,x,y,z,dt)
        implicit double precision (a-h,o-z)
        dimension a(38)

c  ----------------------------------------------------------------------
c       Applying the GEOMETRIC ALGORITHM, this subroutine calculates
c       the coordinates (WGS-84) of a satellite at the "emission" epoch of
c       the signal, and in the system tied to the Earth at the "reception"
c       epoch, from the reception epoch (iyear,idoy,sec --GPS-time--)
c       and navigation message parameters.
c       It also provides the difference between emission and reception
c       epochs.
c
c       NOTE: Any offset in the receiver clock affecting to the reception
c             time, will be translated to the emission time by the
c             geometric algorithm.
c
c       INPUT:
c       -----
c        -  Instant in which the signal is receibed
c            iyear: year
c            idoy:  day of year
c            sec:   seconds in the day
c            (x_sta,y_sta,z_sta): aproximate coordinates of the receiver
c                                 position WGS'84 (in meters)
c        - Navigation message (broadcast data)
c          (according to RINEX format)
c
c            [sat, year,mon,day,h,m,sec, a0,a1,a2     ]
c            [                    IODE,Crs,dn,xMo,    ]
c            [                    Cuc, e, Cus,a12,    ]
c            [                    toe,Cic,Omgg,Cis,   ]
c            [                    xIo,Crc,omgp,Omgd   ]
c            [                    xIDOT,xx,GPS_Week,xx]
c            [                    SVac,SVh,TGD,IODC   ]
c            [                    xx, xx , xx ,xx     ]
c
c      OUTPUT:
c      ------
c             Satellite coordinates at the emission epoch in the
c             reference system tied to the Earth at the "reception"
c             epoch (x,y,z) WGS84 in meters
c
c             dt: t_recepcion - t_emision (seconds)
c
c     NOTE:
c     A simple example about how use this program is shown as follows
c     (program [coord_ems.f]
c
c
cc ========================== coord_ems.f ==============================
cc234567
c      program coord_ems
c      implicit double precision (a-h,o-z)
c      dimension a(38)
c
c
cc ----------------------------------------------------------------------------
cc a) Create a file "data_ems" with the following  data (see "rec2ems.f"
cc    arranged in one single line (fields must be separated with a
cc    blank space!!):
cc
cc 1998 286 38230 4789031  176612 4195008
cc 14 98 10 13 12 0 0 +5.65452501178E-06 +9.09494701773E-13 +0.000000000000E+00
cc +1.28000000000E+02 -6.10000000000E+01 +4.38125402624E-09 +8.198042513605E-01
cc -3.31364572048E-06 +1.09227513894E-03 +5.67547976971E-06 +5.153795101166E+03
cc +2.16000000000E+05 -6.33299350738E-08 +1.00409621952E+00 -3.725290298462E-09
cc +9.73658001335E-01 +2.74031250000E+02 +2.66122811383E+00 -8.081050495434E-09
cc -1.45720352451E-10 +1.00000000000E+00 +9.79000000000E+02 +0.000000000000E+00
cc +3.20000000000E+01 +0.00000000000E+00 -2.32830643654E-09 +1.280000000000E+02
cc +2.08818000000E+05 +0.00000000000E+00 +0.00000000000E+00 +0.000000000000E+00
cc
cc
cc b) Execute:
cc
cc       cat data_ems | coord_ems
cc
cc
cc
cc ----------------------------------------------------------------------------
c
c
c
c      read(*,*,end=10) iyear,idoy,sec,x_sta,y_sta,z_sta,a
c10    continue
c
c      call rec2ems(iyear,idoy,sec,x_sta,y_sta,z_sta,a,x,y,z,dt)
c
c      write(*,'(f4.1,1x,f14.4,1x,f14.4,1x,f14.4,f14.4,1x,f5.3)'),
c     * a(1),sec,x,y,z,dt
c
c      end
c
c
c
c
cc     ----------------- compilacion -------------------
cc      f77 -c orbit.f
cc      f77 -c rec2ems.f
cc      f77 -o coord_ems  coord_ems.f  rec2ems.o orbit.o
cc     -------------------------------------------------
cc ===========================================================================
c     ........................................................
c     ........................................................
c      @gAGE (Research group of Astronomy and GEomatics). UPC
c  --------------------------------------------------------------------


c       CALCULATION OF THE SATELLITE COORDINATES
c       Parameter statement: ...........
c       - Maximum number of iterations
           nit_max=10
c       - Tolerance
           tol=1.d-3   
c       - Velocity of light (m/s)
           c=299792458.d0 
c       - Rotation velocity of the Earth (rad/s)   
          om_e=7.2921151467d-5   
c       ......................................



c   ==> at the RECEPTION instant <==
        call orbit(iyear,idoy,sec,a,x,y,z,Ek)


c   ==> at the EMISSION instant <==
        nit=0
        sec1=sec

60      continue
c       ... Geometric distance (in time) satellite-station
        dt=dsqrt((x-x_sta)**2+(y-y_sta)**2+(z-z_sta)**2)/c
        sec=sec1-dt

c       .... coordinate calculation for "t:=sec1-dt"
        call orbit(iyear,idoy,sec,a,x0,y0,z0,Ek)

        ctl=dsqrt((x0-xa)**2+(y0-ya)**2+(z0-za)**2)      
    
        xa=x0
        ya=y0
        za=z0
        
c       Coordinate transformation to the system tied to the Earth
c       at the "recepcion" instant
c       .... Earth rotation during the time "dt" 
        x=x0+y0*om_e*dt
        y=y0-x0*om_e*dt
        z=z0
c       ...................................................                

        nit=nit+1
        if (ctl.gt.tol) then
          if (nit.lt.nit_max) then
              goto 60
            else
              print *, "ERROR: the algorithm does not converge ",nit
           endif
        endif                                                               

        return
        end



c       f77 -c  rec2ems.f
